<?php
/**
* Gestionnaire pour l'authentification, l'autorisation et les proriete d'un utilisateur 
*
* PHP versions 5
* 
* @category  PHP
* @package   Borealis-security  
* @author    Pascal Martin 
* @copyright Boreal Informations Strategiques Inc.
* @license   http://www.boreal-is.com/license Borealis License
* @link      http://www.boreal-is.com
*/

/**
* Gestionnaire pour l'authentification, l'autorisation et les proriete d'un utilisateur
* 
* @category   PHP 
* @package    Borealis-security
* @subpackage Manager
* @author     Pascal Martin
*/
class com_borealis_security_manager_SecurityManager implements com_borealis_foundation_util_Singleton
{
    private static $s_objInstance = null;
 
    /**
    * @var com_borealis_security_user_User
    */
    private $m_objUser = null;
 
    /**
    * Construit une nouvelle instance de com_borealis_security_manager_SecurityManager
    * 
    * @return com_borealis_security_manager_SecurityManager
    */
    private function __construct()
    {
        $objAuthentificationManager = com_borealis_security_manager_authentication_AuthenticationManagerFactory::getAuthentificationManager();
        // @codeCoverageIgnoreStart 
        if ($objAuthentificationManager->getSession() != null)
        {
            // Load user
            $this->m_objUser = $this->makeUser($objAuthentificationManager->getSession());              
        }
        else
        {
            // Clear user 
            $this->setUser(null);
        } 
        // @codeCoverageIgnoreEnd        
    }

    /**
    * Return the unique instance of class com_borealis_security_manager_SecurityManager 
    * 
    * @return com_borealis_security_manager_SecurityManager 
    */
    public static function getInstance()
    {
        if (is_null(self::$s_objInstance))
        {
            self::$s_objInstance = new com_borealis_security_manager_SecurityManager();
        }
        return self::$s_objInstance;
    } 
    
    /**
    * Return l'utilisateur presentement authentifier or null
    * 
    * @return com_borealis_security_user_User
    */
    public function getUser()
    {
         return $this->m_objUser;        
    }
    
    /**
    * Set l'utilisateur presentement authentifier 
    * 
    * @param com_borealis_security_user_User $pobjUser User
    * 
    * @return void
    */
    private function setUser(com_borealis_security_user_User $pobjUser = null)
    {
        $this->m_objUser = $pobjUser;    
        self::setUserInStore($pobjUser);
    }
    
    /**
    * Log un utilisateur, authentification
    * 
    * @param string                                  $pstrUsername Nom d'utilisateur
    * @param com_borealis_security_password_Password $pobjPassword Mot de passe
    * 
    * @throws com_borealis_security_exception_HostNotFoundException
    * @throws com_borealis_security_exception_ServiceNotFoundException
    * @throws com_borealis_security_exception_UnknownUserException
    * 
    * @return com_borealis_security_user_User
    */
    public function login($pstrUsername, com_borealis_security_password_Password $pobjPassword)
    {
        $this->setUser(null);           
        $objAuthentificationManager = com_borealis_security_manager_authentication_AuthenticationManagerFactory::getAuthentificationManager();
        $objSession = $objAuthentificationManager->login($pstrUsername, $pobjPassword);
                
        return $this->makeUser($objSession);
    }
    
    /**
    * Log un utilisateur avec une session deja ouverte  
    * 
    * @param com_borealis_security_session_SSOSession $pobjSSOSession Target session
    * 
    * @throws com_borealis_security_exception_HostNotFoundException
    * @throws com_borealis_security_exception_ServiceNotFoundException
    * @throws com_borealis_security_exception_BadSessionException
    * 
    * @return com_borealis_security_user_User
    */
    public function loginWithSSOSession(com_borealis_security_session_SSOSession $pobjSSOSession)
    {
        $this->setUser(null);
        $objAuthentificationManager = com_borealis_security_manager_authentication_AuthenticationManagerFactory::getAuthentificationManager();
        $objSession = $objAuthentificationManager->loginWithSSOSession($pobjSSOSession);
        
        return $this->makeUser($objSession);        
    }
       
    // @codeCoverageIgnoreStart To dificult to test because of use of header 
    /**
    * Log un utilisateur avec la passerelle
    * 
    * @return com_borealis_security_user_User   
    */
    public function loginWithGateway()
    {
        $this->setUser(null);
        $objAuthentificationManager = com_borealis_security_manager_authentication_AuthenticationManagerFactory::getAuthentificationManager();
        $objSession = $objAuthentificationManager->loginWithGateway();  
        if ($objSession !== null)
        { 
            return $this->makeUser($objSession);
        }
        else
        {
            return null;  
        }
    }
    // @codeCoverageIgnoreEnd
    
    /**
    * Creer un nouveau user a partir de la session
    * 
    * @param com_borealis_security_session_SSOSession $pobjSession Session du user
    * 
    * @return com_borealis_security_user_User
    */
    private function makeUser(com_borealis_security_session_SSOSession $pobjSession)
    {
        $objAuthorizationManager = com_borealis_security_manager_authorization_AuthorizationManagerFactory::getAuthorizationManager();    
        $objArrayRoles = $objAuthorizationManager->getRoles($pobjSession);
        
        $objPropertyManager = com_borealis_security_manager_property_PropertyManagerFactory::getPropertyManager();    
        $objArrayProperties = $objPropertyManager->getProperties($pobjSession);
            
        $this->setUser(new com_borealis_security_user_User($pobjSession, $objArrayRoles, $objArrayProperties));        
        
        return $this->getUser();
    }
    
    /**
    * Fermeture de la session du user courant    
    * 
    * @throws com_borealis_security_exception_HostNotFoundException
    * @throws com_borealis_security_exception_ServiceNotFoundException
    * 
    * @return void
    */
    public function logout()
    {
        if ($this->getUser() === null)
        {
            return;
        }
        $this->setUser(null);
        $objAuthentificationManager = com_borealis_security_manager_authentication_AuthenticationManagerFactory::getAuthentificationManager();
        $objAuthentificationManager->logout();        
    } 
    
    // @codeCoverageIgnoreStart
    /**
    * Return user from cookie
    * 
    * @return com_borealis_security_user_User
    */
    private static function getUserFromStore()
    {
        if (!isset($_COOKIE['BOREALIS_SECURITY_USER']))
        {
            self::setUserInStore(null);      
        }
        return unserialize($_COOKIE['BOREALIS_SECURITY_USER']);     
    }
    // @codeCoverageIgnoreEnd
   
    /**
    * Save user in cookie
    *  
    * @param com_borealis_security_user_User $pobjUser User
    * 
    * @return void
    */
    private static function setUserInStore(com_borealis_security_user_User $pobjUser = null)
    {
        $_COOKIE['BOREALIS_SECURITY_USER'] = serialize($pobjUser);  
        setcookie('BOREALIS_SECURITY_USER', $_COOKIE['BOREALIS_SECURITY_USER'], 0, "/"); // session cookie ...     
    }

}    

?>
