<?php
/**
* Classe de base pour les objects Action/Ressources/Role    
*
* PHP versions 5
* 
* @category  PHP
* @package   Borealis-acl  
* @author    Pascal Martin 
* @copyright 2009, Boreal - Information Strategies
* @license   http://www.boreal-is.com/license BSD License
* @link      http://www.boreal-is.com
*/

/**
* Classe de base pour les objects Action/Ressources/Role
* 
* @category   PHP 
* @package    Borealis-acl
* @subpackage Object  
* @author     Pascal Martin 
*/ 
abstract class com_borealis_acl_object_Base
{
    /**
    * @var string
    */
    private $m_strName;
    
    /**
    * @var int
    */
    private $m_iID; 
        
    /**
    * Construit une nouvelle instance de com_borealis_acl_object_Base 
    * 
    * @param integer $piID     Identifiant de l'objet
    * @param string  $pstrName Nom de l'objet
    * 
    * @throws com_borealis_foundation_exception_AssertException
    * 
    * @return com_borealis_acl_object_Base
    */
    protected function __construct($piID, $pstrName) 
    {
        com_borealis_foundation_util_Assert::assertIsInteger($piID);
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrName);
        
        $this->m_iID = $piID;
        $this->m_strName = $pstrName;
    }

    /**
    * Retourne la valeur de l'identifient de l'objet
    * 
    * @return string
    */
    public function getID()
    {
        return $this->m_iID;        
    } 
    
    /**
    * Retourne le nom de l'objet
    * 
    * @return string
    */
    public function getName()
    {
        return $this->m_strName;        
    }   
    
    /**
    * Change le nom de l'objet
    * 
    * @param string $pstrName Nom de l'objet
    * 
    * @return void
    * 
    */
    public function setName($pstrName)
    {
        com_borealis_acl_datastore_DataStoreFactory::getDataStore()->update($this, $pstrName);
        $this->m_strName = $pstrName;        
    }   
   
    /**
    * Supprime l'objet
    * 
    * @return void
    */
    public function remove()
    {
        com_borealis_acl_datastore_DataStoreFactory::getDataStore()->deleteByID($this);   
    }
    
    /**
    * Create new objet and return
    * 
    * @param string $pstrType Type de l'objet
    * @param string $pstrName Nom de l'objet
    * 
    * @return int
    */
    protected static function create($pstrType, $pstrName)
    {
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrName);
        
        $iID = com_borealis_acl_datastore_DataStoreFactory::getDataStore()->insert($pstrType, $pstrName);
        return self::newObject($pstrType, $iID, $pstrName);
    }
      
    /**
    * Return object id if exist else null
    * 
    * @param string $pstrType Type de l'objet
    * @param string $pstrName Nom de l'objet
    * 
    * @return int
    */
    protected static function getByName($pstrType, $pstrName)
    {
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrName);
        
        $iID = com_borealis_acl_datastore_DataStoreFactory::getDataStore()->selectByName($pstrType, $pstrName);            
        return self::newObject($pstrType, $iID, $pstrName);
    }   
    
    /**
    * Return object name if exist else null
    * 
    * @param string $pstrType Type de l'objet
    * @param int    $piID     Indentificateur de l'objet
    * 
    * @return string
    */
    protected static function getByID($pstrType, $piID)
    {
        com_borealis_foundation_util_Assert::assertIsInteger($piID);
        
        $strName = com_borealis_acl_datastore_DataStoreFactory::getDataStore()->selectByID($pstrType, $piID);    
        return self::newObject($pstrType, $piID, $strName); 
    } 
    
    /**
    * Return all ressource
    * 
    * @param string $pstrType Type de l'objet
    * 
    * @return ArrayObject of com_borealis_acl_object_Base 
    */
    protected static function getAll($pstrType)
    {
        $objArrayInfos = com_borealis_acl_datastore_DataStoreFactory::getDataStore()->selectAll($pstrType);    
        $objArrayAll = new ArrayObject();
        
        foreach ($objArrayInfos as $objArrayInfo)
        {
            $objArrayAll->append(self::newObject($pstrType, (int)$objArrayInfo['id'], $objArrayInfo['name'])); 
        }
        
        return $objArrayAll;
    }
    
    /**
    * Transform un array de IDName en array d'object
    * 
    * @param string      $pstrType       Type de l'objet
    * @param ArrayObject $pobjArrayInfos Array of id and name
    * 
    * @return ArrayObject
    */
    protected static function getArrayObjectFromArrayIDName($pstrType, ArrayObject $pobjArrayInfos)
    {
        $objArrayResult = new ArrayObject();
        foreach ($pobjArrayInfos as $objArrayInfo)
        {
            $objArrayResult->append(self::newObject($pstrType, (int)$objArrayInfo['id'], $objArrayInfo['name'])); 
        }
        return $objArrayResult;      
    }
    
    /**
    * Make new object after param check
    * 
    * @param string $pstrType Type de l'objet
    * @param int    $piID     Indentificateur
    * @param string $pstrName Nom
    * 
    * @return com_borealis_acl_object_Base
    */
    private static function newObject($pstrType, $piID, $pstrName)
    {
        if ($piID === null || $pstrName === null)
        {
            return null;
        }   
        else
        {   
            $strClassname = str_replace('Base', $pstrType, __CLASS__);                                         
            return new $strClassname($piID, $pstrName);
        }   
    }
}
  
?>
