<?php
/**
* Abstract class for proxies 
*
* PHP versions 5
* 
* @category  PHP
* @package   GeoPrisma  
* @author    Pascal Martin 
* @copyright 2009, Boreal - Information Strategies
* @license   http://www.geoprisma.org/license BSD License
* @link      http://www.geoprisma.org
*/

/**
* Abstract class for proxies 
* 
* @category   PHP 
* @package    GeoPrisma
* @subpackage Proxy 
* @author     Pascal Martin 
*/ 
abstract class org_geoprisma_proxy_Proxy
{
    const CRUD_CREATE = 'create';
    const CRUD_READ   = 'read'; 
    const CRUD_UPDATE = 'update'; 
    const CRUD_DELETE = 'delete'; 
    
    /**
    * @var org_geoprisma_service_Service
    */
    private $m_objService = null;

    /**
    * @var org_geoprisma_service_Resource
    */
    private $m_objResource = null;

    /**
    * Contruist un instance de org_geoprisma_proxy_Proxy
    * 
    * @param org_geoprisma_service_Service $pobjService Service
    * 
    * @return org_geoprisma_proxy_Proxy
    */
    public function __construct(org_geoprisma_service_Service $pobjService) 
    {
        $this->m_objService = $pobjService; 
    }

    // --------------------------
    // Getter
    // --------------------------
    
    /**
    * Returns the proxy's service
    * 
    * @return ArrayObject
    */
    protected function getService()
    {
        return $this->m_objService;   
    }

    /**
    * Get resources of the proxy
    * 
    * @return ArrayObject
    */
    protected function getResources()
    {
        return $this->m_objResource;   
    }

    
    // ###############################
    // ## Abstract function
    // ###############################
    
    /**
    * Returns the name of the action done by this proxy (Create / Read / Update / Delete)
    *                                   
    * @return string
    */
    public abstract function getAction();    
    
    /**
    * Foward the call end return the result
    * 
    * @return void
    */
    public abstract function process();
    
    /**
    * Returns the list of layers accessible by this proxy
    * 
    * @return ArrayObject of string
    */
    public abstract function getLayers();
    
    // ###############################
    // ## Utility functions for subclasses
    // ###############################
    
    /**
    * Ajoute les paramètre recus en get a l'url
    * 
    * @param string $pstrUrl Url
    * 
    * @return string
    */
    static public function addParam($pstrUrl)
    {
        if (strpos($pstrUrl, '?') === false) 
        {
            return $pstrUrl."?".http_build_query(self::getRequestParams());
        }
        else
        {
            return $pstrUrl."&".http_build_query(self::getRequestParams());
        }       
    }
    
    /**
    * Return $_SERVER['PATH_INFO'] or empty string 
    * 
    * @return string    
    */
    static public function getPathInfo()
    {
        if (isset($_SERVER['PATH_INFO'])) 
        {
            return $_SERVER['PATH_INFO'];
        }
        return ''; 
    }

    /**
    * Echo the encoded json result of a proxy query.
    *
    * @param object $pobjResult The result value of the query.
    * 
    * @return void    
    */
    static public function echoJsonResult($pobjResult)
    {
        echo json_encode(array("result" => $pobjResult));
    }

    /**
    * Echo the encoded json error that occured during a proxy query.
    * 
    * @param string $pstrError The error message
    *
    * @return void    
    */
    static public function echoJsonError($pstrError)
    {
        echo json_encode(array("error" => $pstrError));
    }

    /**
    * Returns all 'non geoprisma' parameters inside $_GET.
    * 
    * @return array    
    */
    static public function getRequestParams()
    {
        $objArrayParams = Array();

        foreach ($_GET as $strKey => $strParam) 
        {
            if (substr($strKey, 0, 3) != "osm") 
            {
                $objArrayParams[$strKey] = $strParam;
            }
        }

        return $objArrayParams;
    }

    /**
    * Set the proxy resource ArrayObject.
    * 
    * @param ArrayObject $pobjArrayResources ArrayObject of resources being
    *                                        accessed by the query.
    *
    * @return void    
    */
    public function setResources($pobjArrayResources)
    {
        if (is_array($pobjArrayResources)) 
        {
            $this->m_objResource = new ArrayObject($pobjArrayResources);
        }
        else
        {
            $this->m_objResource = $pobjArrayResources;
        }

    }

    /**
    * Returns an array of org_geoprisma_resource_Resource objects being accessed
    *     by the query.
    * 
    * @param org_geoprisma_config_Config &$pobjConfig The config object
    * @param array                       $pobjRequest The request array that
    *                                                 contains the 'osmservice'
    *                                                 and 'osmresource'
    *                                                 parameters.  Defaults to
    *                                                 $_REQUEST.
    *
    * @return array    
    */
    public function getResourcesFromRequest(&$pobjConfig, $pobjRequest=null)
    {
        $objRequest = ($pobjRequest) ? $pobjRequest : $_REQUEST;

        if (!isset($objRequest['osmservice'])) 
        {
            throw new Exception("osmservice param is missing");
        }

        $objArrayResource = Array();
        if (is_array($objRequest['osmresource'])) 
        {
            foreach ($objRequest['osmresource'] as $strRessourceName)
            {
                array_push(
                    $objArrayResource,
                    $pobjConfig->getResource($strRessourceName)
                );
            } 
        }
        else
        {
            array_push(
                $objArrayResource,
                $pobjConfig->getResource($objRequest['osmresource'])
            );
        }

        return $objArrayResource;
    }

    /**
    * Validate the resources being accessed by the query by simply loading the
    *     datastore of a specified service.
    * 
    * @param org_geoprisma_service_Service $pobjService        The service to
    *                                                          use for 
    *                                                          validation.
    *                                                          Defaults to
    *                                                          the proxy
    *                                                          service.
    * @param ArrayObject                   $pobjArrayResources ArrayObject of
    *                                                          resource objects
    *                                                          to validate.
    *                                                          Defaults to the
    *                                                          proxy resources.
    *
    * @return void    
    */
    public function validateResourcesFromRequest(
        $pobjService=null,
        $pobjArrayResources=null
    ) {
        $objService = ($pobjService === null) ? $this->getService() : $pobjService;
        $objArrayResources = ($pobjArrayResources === null) ? $this->getResources() : $pobjArrayResources;
        
        foreach ($objArrayResources as $objResource)
        {
            if ($objResource->getDatastore($objService) === null) 
            {
                throw new Exception('Not Authorized by Config - Service');   
            }
        }
    }

    /**
    * Validate the 'layers' being accessed by the query.  Each layer must be
    *     present in one of the specified resource datastore using a specified
    *     service.
    * 
    * @param org_geoprisma_service_Service $pobjService        The service to
    *                                                          use for 
    *                                                          validation.
    *                                                          Defaults to
    *                                                          the proxy
    *                                                          service.
    * @param ArrayObject                   $pobjArrayResources ArrayObject of
    *                                                          resource objects.
    *                                                          Defaults to the
    *                                                          proxy resources.
    * @param ArrayObject                   $pobjArrayLayers    ArrayObject of
    *                                                          layers to
    *                                                          validate.
    *                                                          Defaults to the
    *                                                          layers returned
    *                                                          by the proxy
    *                                                          'getLayers'
    *                                                          method. 
    *
    * @return void    
    */
    public function validateLayersFromRequest(
        $pobjService=null,
        $pobjArrayResources=null,
        $pobjArrayLayers=null
    ) {
        $objService = ($pobjService === null) ? $this->getService() : $pobjService;
        $objArrayResources = ($pobjArrayResources === null) ? $this->getResources() : $pobjArrayResources;
        $objArrayLayers = ($pobjArrayLayers === null) ? $this->getLayers() : $pobjArrayLayers;


        $bAllAuthorized = true;
        foreach ($objArrayLayers as $strLayer) 
        {
            $bIsAuthorized = false;
            foreach ($objArrayResources as $objResource) 
            {
                $objDatastore = $objResource->getDatastore($objService);
                if ($objDatastore->isAuthorizedLayer($strLayer)) 
                {
                    $bIsAuthorized = true;
                    break;
                }
            }
            if (!$bIsAuthorized) 
            {
                $bAllAuthorized = false;
                break;
            }
        }

        if (!$bAllAuthorized) 
        {
            throw new Exception('Not Authorized by Datastore layer');    
        }
    }

    /**
    * Validate that the resources accessed by the query are authorized by the
    *     ACL using the proxy action.
    * 
    * @param ArrayObject $pobjArrayResources ArrayObject of resource objects.
    *                                        Defaults to the proxy resources.
    *
    * @return void    
    */
    public function areResourcesFromRequestAuthorized($pobjArrayResources=null)
    {
        $objArrayResources = ($pobjArrayResources === null) ? $this->getResources() : $pobjArrayResources;

        foreach ($objArrayResources as $objResource) 
        {
            if (!org_geoprisma_acl_ACLFactory::getACL()->isAuthorized($objResource, $this->getAction())) 
            {
                throw new Exception('Not Authorized by ACL');   
            }
        }
    }

    /**
    * Validate and execute all the access filters.
    * The format of a condition string is always:
    *    ISSET##input##true_behavior##false_behavior
    *    BBOX##input##true_behavior##false_behavior
    * 
    * @param ArrayObject $pobjArrayResources ArrayObject of resource objects.
    *                                        Defaults to the proxy resources.
    *
    * @return void    
    */
    public function applyAccessFilter($pobjService, $pobjArrayResources=null)
    {
        $objArrayResources = ($pobjArrayResources === null) ? $this->getResources() : $pobjArrayResources;

        foreach ($objArrayResources as $objResource) 
        {
            $objArrayAccessFilter = $objResource->getAccessFilters();
            foreach($objArrayAccessFilter as $objAccessFilter)
            {
                $objArrayConditions = $objAccessFilter->getConditions();
                foreach($objArrayConditions as $strCondition)
                {
                    $strConditionType = '';
                    $strInput = '';
                    $strTrueBehavior = '';
                    $strFalseBehavior = '';

                    $objArrayConditionParts = explode('##', $strCondition);

                    // Need at least 3 parts
                    if(count($objArrayConditionParts) < 3)
                        next;

                    $strConditionType = $objArrayConditionParts[0];
                    $strInput = $objAccessFilter->readInput($objArrayConditionParts[1]);
                    $strTrueBehavior = $objArrayConditionParts[2];  
                    if(count($objArrayConditionParts) >= 4)
                        $strFalseBehavior = $objArrayConditionParts[3];

                    $pobjService->validateAccessFilterCondition(
                        $strConditionType, 
                        $strInput, 
                        $strTrueBehavior, 
                        $strFalseBehavior
                    );
                }
            }
        }
    }
}
  
?>
