<?php
/**
* Abstract class for widgets fonctionnalities
*
* PHP versions 5
* 
* @category  PHP
* @package   GeoPrisma  
* @author    Stephane Guillemette
* @copyright 2009, Boreal - Information Strategies
* @license   http://www.geoprisma.org/license BSD License
* @link      http://www.geoprisma.org
*/

/**
* Abstract class for widgets fonctionnalities
*  
* @category   PHP 
* @package    GeoPrisma
* @subpackage UI
* @author     Stephane Guillemette
* @license    http://www.geoprisma.org/license BSD License
* @link       http://www.geoprisma.org
*/
  
abstract class org_geoprisma_widget_WidgetBase
{
    /**
    * Return text based on domainName for right language
    * 
    * @param string $pstrMsg        Message Key
    * @param string $pstrDomainName Name of the
    * @param string $pstrType       Type encoding "JS" "HTML" 
    *
    * @return string 
    */  
    static public function getText($pstrMsg, $pstrDomainName, $pstrType = null)
    {
        return org_geoprisma_locale_Locale::getText(
            $pstrMsg, $pstrDomainName, $pstrType
        );
    }
    
    /**
    * Retourne l'action du widget (pour acl)
    * 
    * @return string
    */ 
    abstract static public function getAction();  
    
    /**
    * Retourne le type du widget (e.q. map, quickzoom, ..)
    * 
    * @return string
    */
    abstract static public function getType();

    /**
    * Returns the widget 'servicetype' it uses.
    * 
    * @return string
    */
    abstract static public function getServiceType();

    /**
    * Wether the widget needs to interact with a layer or not.  Defaults to
    *     false.
    * 
    * @return boolean
    */
    static public function needLayer() 
    {
        return false;
    }

    /**
    * Return the list of Resource options a Widget needs to read in order to
    *     do something.  If the Widget doesn't need to be linked to any Resource
    *     this method should return NULL (which is the default value).
    * 
    * @return array
    */
    static public function getMandatoryResourceOptionList()
    {
        return null;
    }

    /**
    * Return the list of optional Resource options a Widget can also use.
    * 
    * @return array
    */
    static public function getOptionalResourceOptionList()
    {
        return null;
    }

    /**
    * Return the list of processes available to the widget.
    * 
    * @return array
    */
    static public function getProcessList()
    {
        return array();
    }

    /**
    * Return the list of mandatory Resource option values a Resource must have
    *     in order to be linked to a widget.  This method needs to be
    *     overwritten by the widget in order to do something.  By default, an
    *     empty array is returned, meaning that the widget doesn't need to have
    *     specific resource option values in order to be linked.
    * 
    * @return array
    */
    static public function getMandatoryResourceOptionValueList() 
    {
        return array();
    }

    /**
    * Return the complete list of Resource options a Widget can use.
    *
    * @param string $pstrType Type of widget
    * 
    * @return array
    */
    static public function getAllResourceOptionList($pstrType)
    {
        $objArrayMandatoryOptions = call_user_func(
            array($pstrType, 'getMandatoryResourceOptionList')
        );

        if ($objArrayMandatoryOptions == null) 
        {
            return $objArrayMandatoryOptions;
        }

        $objArrayOptionalOptions = call_user_func(
            array($pstrType, 'getOptionalResourceOptionList')
        );

        if ($objArrayOptionalOptions != null) 
        {
            $objArrayMandatoryOptions = array_merge(
                $objArrayMandatoryOptions, $objArrayOptionalOptions
            );
        }

        return $objArrayMandatoryOptions;
    }

    /**
    * Wheter the widget needs to be cloned when generating the 'config_secur'
    *     dom while using new configs that separate widgets from resources.
    * 
    * @return boolean
    */
    static public function mustClone()
    {
        return false;
    }

    /**
    * Returns an array of options that are in resource using the
    *     'getAllResourceOptionList' method.
    *
    * @param org_geoprisma_resource_Resource $pobjResource Resource object
    * @param string                          $pstrType     Type of widget
    * 
    * @return array of org_geoprisma_option_Option objects
    */
    static public function getOptionsFromResource($pobjResource, $pstrType) 
    {
        $objArrayOptions = array();

        $objArrayOptionList = call_user_func_array(
            array($pstrType, 'getAllResourceOptionList'),
            array($pstrType)
        );

        if ($objArrayOptionList != null) 
        {
            foreach ($objArrayOptionList
                     as $strWidgetOption => $strResourceOption)
            {
                $objOption = $pobjResource->getOption($strResourceOption, true);
                if ($objOption !== false) 
                {
                    array_push(
                        $objArrayOptions,
                        new org_geoprisma_option_Option(
                            $strWidgetOption, $objOption->getValue()
                        )
                    );
                }
            }
        }

        return $objArrayOptions;
    }

    /**
    * When linking a widget and a resource together, sometimes we need to 
    *     add custom elements to the widgets using the resource options.  This
    *     method is called and used for this purpose.
    *
    * Since is unique for each widget, it needs to be overwritten by each
    *     widget if it wants to actually do something.  Does nothing by default.
    *
    * @param org_geoprisma_resource_Resource $pobjResource Resource object
    * @param org_geoprisma_widget_Widget     $pobjWidget   Widget object
    * @param org_geoprisma_config_Config     $pobjConfig   Config object
    * 
    * @return void
    */
    static public function linkResource(
        $pobjResource=null, $pobjWidget=null, $pobjConfig=null
    ) {
        if ($pobjResource == null || $pobjWidget == null) 
        {
            return;
        }
    }
}
  
?>
